#!/usr/bin/env python3
__package__ = __package__ or "tests.engines"  # pylint:disable=redefined-builtin

import os
import unittest

import angr
from angr.storage.memory_mixins import JavaVmMemory, DefaultMemory, KeyValueMemory
from angr.engines.soot.values import SimSootValue_ArrayRef, SimSootValue_ThisRef
from angr.engines.soot.method_dispatcher import resolve_method
from archinfo.arch_amd64 import ArchAMD64
from archinfo.arch_soot import (
    ArchSoot,
    SootAddressDescriptor,
    SootMethodDescriptor,
    SootArgument,
    SootAddressTerminator,
)
from claripy.backends.backend_smtlib_solvers import z3str_popen  # noqa: F401

try:
    import pysoot
except ModuleNotFoundError:
    pysoot = None

from ..common import bin_location


test_location = os.path.join(bin_location, "tests", "java")


@unittest.skipUnless(pysoot, "pysoot not available")
def test_fauxware():
    # create project
    binary_path = os.path.join(test_location, "fauxware_java_jni", "fauxware.jar")
    jni_options = {"jni_libs": ["libfauxware.so"]}
    project = angr.Project(binary_path, main_opts=jni_options)
    entry = project.factory.entry_state()
    simgr = project.factory.simgr(entry)

    # find path to `accepted()` method
    accepted_method = SootMethodDescriptor.from_string("Fauxware.accepted()").address()
    simgr.explore(find=lambda s: s.addr == accepted_method)

    state = simgr.found[0]

    # eval password
    cmd_line_args = project.simos.get_cmd_line_args(state)
    password = state.solver.eval(cmd_line_args[0])
    assert password == "SOSNEAKY"


def test_apk_loading():
    sdk_path = os.path.join(os.path.expanduser("~"), "Android/Sdk/platforms/")
    if not os.path.exists(sdk_path):
        print("cannot run test_apk_loading since there is no Android SDK folder")
        return

    loading_opts = {
        "android_sdk": sdk_path,
        "entry_point": "com.example.antoniob.android1.MainActivity.onCreate",
        "entry_point_params": ("android.os.Bundle",),
    }
    project = angr.Project(os.path.join(test_location, "android1.apk"), main_opts=loading_opts, auto_load_libs=False)

    blank_state = project.factory.blank_state()
    a1 = SimSootValue_ThisRef.new_object(blank_state, "com.example.antoniob.android1.MainActivity")
    a2 = SimSootValue_ThisRef.new_object(blank_state, "android.os.Bundle", symbolic=True)
    args = [SootArgument(arg, arg.type) for arg in [a1, a2]]
    entry = project.factory.entry_state(args=args)

    simgr = project.factory.simgr(entry)
    simgr.step()
    simgr.step()
    assert simgr.active[0].addr.block_idx == 0
    assert simgr.active[0].addr.stmt_idx == 3
    simgr.run()
    assert len(simgr.deadended) == 1
    assert type(simgr.deadended[0].addr) is SootAddressTerminator


#
# Command line arguments
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_cmd_line_args():
    project = create_project("cmd_line_args", load_native_libs=False)
    entry = project.factory.entry_state()
    simgr = project.factory.simgr(entry)
    simgr.run()
    assert len(simgr.deadended) == 2
    state1, state2 = tuple(simgr.deadended)

    # get symbol of args[0] from memory
    args = state1.globals["cmd_line_args"]
    args0_arrref = SimSootValue_ArrayRef(args, 0)
    args0_strref = state1.memory.load(args0_arrref)
    args0_strval = state1.memory.load(args0_strref)

    # eval args[0] on both states
    str1 = state1.solver.eval(args0_strval)
    str2 = state2.solver.eval(args0_strval)
    assert "secret_value" in [str1, str2]


#
# JNI Version Information
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_jni_version_information():
    project = create_project("jni_version_information")

    run_method(project=project, method="MixedJava.test_jni_get_version", assert_locals={"i0": 0x10008})


#
# JNI Global and Local References
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_jni_global_and_local_refs():
    project = create_project("jni_global_and_local_refs")

    assertions = {"global refs dict": lambda state: (state.jni_references.global_refs == {})}
    run_method(
        project=project, method="MixedJava.test_jni_global_refs", assert_locals={"i0": 0xA}, assertions=assertions
    )


#
# JNI Object Operations
#


def test_jni_object_operations():
    project = create_project("jni_object_operations")

    run_method(project=project, method="MixedJava.test_jni_alloc_object", assert_locals={"i0": 0})

    run_method(project=project, method="MixedJava.test_jni_new_object", assert_locals={"i0": 1})

    run_method(project=project, method="MixedJava.test_jni_new_subclass_object", assert_locals={"i0": 2})

    run_method(
        project=project, method="MixedJava.test_jni_isinstanceof", assert_locals={"i0": 1, "i1": 1, "i2": 0, "i3": 1}
    )

    run_method(project=project, method="MixedJava.test_jni_issameobject", assert_locals={"i0": 0, "i1": 1})


#
# JNI String Operations
#


def test_jni_string_operations():
    project = create_project("jni_string_operations")

    assertions = {
        "1st string": lambda state: (state.solver.eval_one(load_string(state, "r0")) == "mum"),
        "2nd string": lambda state: (state.solver.eval_one(load_string(state, "r1")) == "himum!"),
    }
    run_method(
        project=project,
        method="MixedJava.test_jni_string_operations",
        assert_locals={"i0": 0x3, "i1": 0x6},
        assertions=assertions,
    )


#
# JNI Field Access
#


def test_jni_field_access():
    project = create_project("jni_field_access")

    run_method(
        project=project,
        method="MixedJava.test_static_field_access_basic",
        assert_locals={"i0": 0x0, "i1": 0x1, "i2": 0xA, "i3": 0xB, "i4": 0x7, "i5": 0xB, "i6": 0x0, "i7": 0x9},
    )

    run_method(project=project, method="MixedJava.test_jni_static_field_access", assert_locals={"i0": 0, "i1": 5})

    run_method(
        project=project,
        method="MixedJava.test_jni_static_field_access_subclass",
        assert_locals={"i0": 1, "i1": 10, "i2": 30, "i3": 1},
    )

    run_method(
        project=project,
        method="MixedJava.test_instance_field_access_0",
        assert_locals={"i0": 0, "i1": 10, "i2": 5, "i3": 5},
    )

    run_method(
        project=project,
        method="MixedJava.test_instance_field_access_1",
        assert_locals={"i0": 0, "i1": 1, "i2": 10, "i3": 4, "i4": 4, "i5": 1},
    )


#
# JNI Method Calls
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_jni_method_calls():
    project = create_project("jni_method_calls")

    run_method(project=project, method="MixedJava.test_jni_non_virtual_instance_method_call", assert_locals={"i0": 5})

    run_method(
        project=project, method="MixedJava.test_jni_instance_method_calls_basic", assert_locals={"i0": 7, "i1": 7}
    )

    run_method(
        project=project, method="MixedJava.test_jni_instance_method_calls_subclass", assert_locals={"i0": 2, "i1": 2}
    )

    run_method(
        project=project,
        method="MixedJava.test_jni_instance_method_calls_shared_method_id",
        assert_locals={"i0": 8, "i1": 2},
    )

    run_method(project=project, method="MixedJava.test_jni_instance_method_calls_args", assert_locals={"i0": 11})

    run_method(project=project, method="MixedJava.test_jni_static_method_call", assert_locals={"i0": 10})

    run_method(project=project, method="MixedJava.test_jni_static_method_call_return_obj", assert_locals={"i0": 7})


#
# JNI Primitive Datatypes
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_jni_primitive_datatypes():
    project = create_project("jni_primitive_datatypes")

    run_method(
        project=project, method="MixedJava.test_boolean", assert_locals={"z0": 1, "z1": 0, "z2": 1, "z3": 0, "z4": 1}
    )

    run_method(project=project, method="MixedJava.test_byte", assert_locals={"b5": 30, "b8": 0xFFFFFF80, "b11": 0})

    run_method(project=project, method="MixedJava.test_char", assert_locals={"c4": 21, "c6": 0, "c9": 1})

    run_method(
        project=project,
        method="MixedJava.test_short",
        assert_locals={"s3": 0x1000, "s5": 0xFFFFF000, "s0": 11, "s9": 0},
    )

    run_method(
        project=project,
        method="MixedJava.test_int",
        assert_locals={"i1": 0xFFFFFFF6, "i3": 0, "i5": 0x80000001, "i7": 0x7FFFFFFF},
    )

    run_method(project=project, method="MixedJava.test_long", assert_locals={"l1": 0xFFFFFFFFFFFFFFFF, "l3": 1})


@unittest.skipUnless(pysoot, "pysoot not available")
def test_jni_object_arrays():
    project = create_project("jni_object_array_operations")

    run_method(project=project, method="MixedJava.test_jni_access_object_array", assert_locals={"i0": 7})

    run_method(project=project, method="MixedJava.test_jni_new_object_array", assert_locals={"i0": 10})


#
# JNI Array Operations
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_jni_array_operations():
    project = create_project("jni_array_operations")

    # test_jni_newarray
    run_method(
        project=project,
        method="MixedJava.test_jni_newarray",
        assert_locals={"i0": 0, "i1": 1, "i2": 2, "i3": 3, "i4": 4},
    )

    # test_jni_getarrayregion
    state = run_method(project=project, method="MixedJava.test_jni_getarrayregion")
    a = load_value_from_stack(state, "i1")
    state.solver.add(a == 15)
    idx = state.posix.stdin.content[0][0]
    assert state.solver.eval_one(idx) == 7

    # test_jni_setarrayregion1
    run_method(
        project=project,
        method="MixedJava.test_jni_setarrayregion1",
        assert_locals={"i0": 0, "i1": 3, "i2": 2, "i3": 1, "i4": 4},
    )

    # test_jni_setarrayregion2
    state = run_method(project=project, method="MixedJava.test_jni_setarrayregion2")
    a = load_value_from_stack(state, "i1")
    state.solver.add(a == 2)
    idx = state.posix.stdin.content[0][0]
    idx_value = state.solver.eval_one(idx)
    assert idx_value == 0

    # test_jni_setarrayregion2
    state = run_method(project=project, method="MixedJava.test_jni_setarrayregion2")
    a = load_value_from_stack(state, "i1")
    state.solver.add(a == 0)
    idx = state.posix.stdin.content[0][0]
    idx_value = state.solver.eval_exact(idx, 2)
    assert 1 in idx_value
    assert 2 in idx_value
    assert 3 not in idx_value

    # test_jni_getarrayelements_symbolic
    winning_path = get_winning_path(project=project, method_fullname="MixedJava.test_jni_getarrayelements_symbolic")
    stdin_packets = winning_path.posix.stdin.content
    idx = winning_path.solver.eval_one(stdin_packets[0][0])
    min_length = winning_path.solver.min(stdin_packets[1][0])
    assert idx == 223
    assert min_length == 224

    # test_jni_releasearrayelements
    run_method(
        project=project,
        method="MixedJava.test_jni_releasearrayelments",
        assert_locals={"i0": 4, "i1": 3, "i2": 2, "i3": 1, "i4": 0},
    )

    # test_jni_getarrayelements_and_releasearrayelements
    run_method(
        project=project,
        method="MixedJava.test_jni_getarrayelements_and_releasearrayelements",
        assert_locals={
            "c9": 0xFFFF,
            "c14": 0x0000,
            "b5": 0x0000007F,
            "b10": 0xFFFFFF80,
            "s6": 0x00007FFF,
            "s11": 0xFFFF8000,
            "i7": 0x7FFFFFFF,
            "i12": 0x80000000,
            "l8": 0x7FFFFFFFFFFFFFFF,
            "l13": 0x8000000000000000,
        },
    )

    # test_jni_getarraylength
    state = run_method(project=project, method="MixedJava.test_jni_getarraylength")
    a = state.memory.stack.load("i3")
    assert state.solver.eval(a) == 10
    b = state.memory.stack.load("i4")
    assert state.solver.min(b) == 0
    assert state.solver.max(b) == 255


#
# Method Calls
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_method_calls():
    project = create_project("method_calls", load_native_libs=False)

    run_method(
        project=project,
        method="MixedJava.test_instance_method_calls",
        assert_locals={"i0": 0, "i1": 1, "i2": 1, "i3": 2, "i4": 2, "i5": 2},
    )

    run_method(
        project=project,
        method="MixedJava.test_static_method_calls_0",
        assert_locals={"i0": 0, "i1": 1, "i2": 2, "i3": 2},
    )

    run_method(
        project=project,
        method="MixedJava.test_static_method_calls_1",
        assert_locals={"i0": 0, "i1": 0, "i2": 1, "i3": 2, "i4": 2, "i5": 2},
    )

    run_method(project=project, method="MixedJava.test_special_invoke_0", assert_locals={"i0": 3})

    run_method(project=project, method="MixedJava.test_special_invoke_1", assert_locals={"i0": 4})


#
# Array Operations
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_array_operations():
    project = create_project("array_operations", load_native_libs=False)

    # test_basic_array_operations
    run_method(
        project=project,
        method="MixedJava.test_basic_array_operations",
        assert_locals={"i1": 0, "i2": 1, "i3": 2, "i4": 3, "i5": 4, "i6": 5, "i7": 2, "i8": 0},
    )

    # test_symbolic_array_read
    winning_path = get_winning_path(project=project, method_fullname="MixedJava.test_symbolic_array_read")
    stdin_packets = winning_path.posix.stdin.content
    input_char, _ = stdin_packets[0]
    solutions = winning_path.solver.eval_upto(input_char, 2)
    assert ord("A") in solutions
    assert ord("C") in solutions

    # test_symbolic_array_write
    winning_path = get_winning_path(project=project, method_fullname="MixedJava.test_symbolic_array_write")
    stdin_packets = winning_path.posix.stdin.content
    idx_symbol, _ = stdin_packets[0]
    val_symbol, _ = stdin_packets[1]
    winning_path.solver.add(val_symbol != 0)  # exclude trivial solution
    idx = winning_path.solver.eval(idx_symbol)
    val = winning_path.solver.eval(val_symbol)
    assert idx == 73
    assert val == 53

    # test_symbolic_array_length
    winning_path = get_winning_path(project=project, method_fullname="MixedJava.test_symbolic_array_length")
    stdin_packets = winning_path.posix.stdin.content
    input_char, _ = stdin_packets[0]
    solution = winning_path.solver.eval(input_char)
    assert solution == ord("F")

    # test_index_of_of_bound0
    state = run_method(project=project, method="MixedJava.test_index_of_of_bound0")
    array_len = load_value_from_stack(state, "i1")
    assert state.solver.min(array_len) == 0
    assert state.solver.max(array_len) == 255

    # test_index_of_of_bound1
    state = run_method(project=project, method="MixedJava.test_index_of_of_bound1")
    array_len = load_value_from_stack(state, "i1")
    assert state.solver.min(array_len) == 101
    assert state.solver.max(array_len) == 255

    # test_index_of_of_bound2
    state = run_method(project=project, method="MixedJava.test_index_of_of_bound2")
    assert load_value_from_stack(state, "i1") is not None
    assert load_value_from_stack(state, "i2") is None
    assert load_value_from_stack(state, "i3") is None
    assert load_value_from_stack(state, "i4") is not None
    assert load_value_from_stack(state, "i5") is None

    # test_index_of_of_bound3
    state = run_method(project=project, method="MixedJava.test_index_of_of_bound3")
    assert load_value_from_stack(state, "i1") is not None
    assert load_value_from_stack(state, "i2") is not None
    assert load_value_from_stack(state, "i3") is None
    assert load_value_from_stack(state, "i4") is not None
    assert load_value_from_stack(state, "i5") is None

    # test_index_of_of_bound4
    state = run_method(project=project, method="MixedJava.test_index_of_of_bound4")
    assert load_value_from_stack(state, "i1") is not None
    assert load_value_from_stack(state, "i2") is not None
    assert load_value_from_stack(state, "i3") is None

    # test_index_of_of_bound5
    state = run_method(project=project, method="MixedJava.test_index_of_of_bound5")
    assert load_value_from_stack(state, "i1") is not None
    assert load_value_from_stack(state, "i2") is not None
    assert load_value_from_stack(state, "i3") is None


#
# MultiArray Operations
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_multiarray_operations():
    project = create_project("multiarray_operations", load_native_libs=False)

    run_method(project=project, method="MixedJava.basic_multiarray_ops", assert_locals={"d1": 4})


#
# Loading
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_loading():
    # Test1: test loading with load path
    native_libs_ld_path = os.path.join(test_location, "misc", "loading1", "libs")
    jar_path = os.path.join(test_location, "misc", "loading1", "mixedjava.jar")
    # define which libraries to load (+ the load path)
    jni_options = {"jni_libs": ["libmixedjava.so"], "jni_libs_ld_path": native_libs_ld_path}
    project = angr.Project(jar_path, main_opts=jni_options, auto_load_libs=True)
    # check if libmixedjava.so was loaded
    loaded_libs = [lib.provides for lib in project.loader.all_elf_objects]
    assert "libmixedjava.so" in loaded_libs

    # Test 2: test loading without load path
    # => the folder of the JAR is implicitly used as an additional load path
    binary_dir = os.path.join(test_location, "misc", "loading2")
    project = create_project(binary_dir)
    # check if libmixedjava.so was loaded
    loaded_libs = [lib.provides for lib in project.loader.all_elf_objects]
    assert "libmixedjava.so" in loaded_libs


#
# SimStates
#


@unittest.skipUnless(pysoot, "pysoot not available")
def test_toggling_of_simstate():
    binary_dir = os.path.join(test_location, "misc", "simstates")
    project = create_project(binary_dir)

    state = project.factory.entry_state()
    assert state.ip_is_soot_addr
    assert isinstance(state.arch, ArchSoot)
    assert isinstance(state.memory, JavaVmMemory)
    assert isinstance(state.registers, KeyValueMemory)

    state.regs.ip = 1
    assert not state.ip_is_soot_addr
    assert isinstance(state.arch, ArchAMD64)
    assert isinstance(state.memory, DefaultMemory)
    assert isinstance(state.registers, DefaultMemory)

    state.regs._ip = project.entry

    assert state.ip_is_soot_addr
    assert isinstance(state.arch, ArchSoot)
    assert isinstance(state.memory, JavaVmMemory)
    assert isinstance(state.registers, KeyValueMemory)

    state.ip = 1
    assert not state.ip_is_soot_addr
    assert isinstance(state.arch, ArchAMD64)
    assert isinstance(state.memory, DefaultMemory)
    assert isinstance(state.registers, DefaultMemory)

    state_copy = state.copy()
    assert not state_copy.ip_is_soot_addr
    assert isinstance(state_copy.arch, ArchAMD64)
    assert isinstance(state_copy.memory, DefaultMemory)
    assert isinstance(state_copy.registers, DefaultMemory)


@unittest.skipUnless(pysoot, "pysoot not available")
def test_object_tracking():
    binary_dir = os.path.join(test_location, "object_tracking")
    project = create_project(binary_dir, load_native_libs=False)
    bootstrap_state = project.factory.blank_state(addr=SootAddressTerminator())
    mylib_object = SimSootValue_ThisRef.new_object(bootstrap_state, "MyLib", symbolic=True, init_object=False)

    soot_method = resolve_method(
        bootstrap_state, "testGetterAndSetterConcrete", "MixedJava", ("mylib.MyLib",), init_class=False
    ).address()

    call_state = project.factory.call_state(
        soot_method,
        SootArgument(mylib_object, mylib_object.type, is_this_ref=False),
        base_state=bootstrap_state,
        ret_addr=SootAddressTerminator(),
    )

    call_state.options.add(angr.options.JAVA_IDENTIFY_GETTER_SETTER)
    call_state.options.add(angr.options.JAVA_TRACK_ATTRIBUTES)

    simgr = project.factory.simgr(call_state)
    simgr.run()

    assert len(simgr.deadended) == 1

    final_state = simgr.deadended[0]

    assert final_state.solver.eval(mylib_object.get_field(final_state, "myInt", "int")) == 1
    assert final_state.solver.eval(mylib_object.get_field(final_state, "myShort", "short")) == 1
    assert final_state.solver.eval(mylib_object.get_field(final_state, "myChar", "char")) == ord("c")
    assert final_state.solver.eval(mylib_object.get_field(final_state, "myLong", "long")) == 2
    assert final_state.solver.eval(mylib_object.get_field(final_state, "myFloat", "float")) == 1.5
    assert final_state.solver.eval(mylib_object.get_field(final_state, "myDouble", "double")) == 1.5
    string_ref = mylib_object.get_field(final_state, "myString", "java.lang.String")
    assert final_state.solver.eval(final_state.memory.load(string_ref)) == "Hello!"
    array_ref = mylib_object.get_field(final_state, "myArray", "int[]")
    assert final_state.solver.eval(array_ref.size) == 3
    object_ref = mylib_object.get_field(final_state, "myObject", "java.lang.Object")
    assert final_state.solver.eval(object_ref.get_field(final_state, "a", "int")) == 1

    assert ("myInt", "int") in mylib_object.attributes
    assert ("myChar", "char") in mylib_object.attributes
    assert ("myShort", "short") in mylib_object.attributes
    assert ("myLong", "long") in mylib_object.attributes
    assert ("myFloat", "float") in mylib_object.attributes
    assert ("myDouble", "double") in mylib_object.attributes
    assert ("myString", "java.lang.String") in mylib_object.attributes
    assert ("myArray", "int[]") in mylib_object.attributes
    assert ("myObject", "java.lang.Object") in mylib_object.attributes


#
# Helper
#


def run_method(project, method, assert_locals=None, assertions=None):
    end_state = get_last_state_of_method(project, method)
    # print_java_memory(end_state)

    if assert_locals:
        for symbol_name, assert_value in assert_locals.items():
            symbol = load_value_from_stack(end_state, symbol_name)
            val = end_state.solver.eval(symbol)
            assert val == assert_value

    if assertions:
        for _, test in assertions.items():
            assert test(end_state)

    return end_state


# def print_java_memory(state):
#     print "\n##### STACK ##########" + "#"*60
#     print state.memory.stack
#     print "\n##### HEAP ###########" + "#"*60
#     print state.memory.heap
#     print "\n##### VM STATIC TABLE " + "#"*60
#     print state.memory.vm_static_table
#     print


@unittest.skipUnless(pysoot, "pysoot not available")
def create_project(binary_dir, load_native_libs=True):
    jar_path = os.path.join(test_location, binary_dir, "mixedjava.jar")
    if load_native_libs:
        jni_options = {"jni_libs": ["libmixedjava.so"]}
        project = angr.Project(jar_path, main_opts=jni_options)
    else:
        project = angr.Project(jar_path)
    return project


def load_string(state, local_name):
    str_ref = load_value_from_stack(state, local_name)
    return state.memory.load(str_ref)


def load_value_from_stack(state, symbol_name):
    try:
        return state.memory.stack.load(symbol_name)
    except KeyError:
        return None


def get_entry_state_of_method(project, method_fullname):
    # get SootAddressDescriptor of method entry
    soot_method = project.loader.main_object.get_soot_method(method_fullname)
    method = SootMethodDescriptor.from_soot_method(soot_method)
    addr = SootAddressDescriptor(method, 0, 0)
    # create call state
    return project.factory.blank_state(addr=addr, add_options={angr.options.ZERO_FILL_UNCONSTRAINED_MEMORY})


def get_last_state_of_method(project, method_fullname):
    state = get_entry_state_of_method(project, method_fullname)
    # run until no successors exists
    # Note: this does not work if conditional branches are present
    states = [state]
    succ = states[-1].step()
    while len(succ.successors) == 1:
        states += succ
        succ = states[-1].step()
    # last state is the 'Terminator' state
    # => return the state before
    return states[-2]


def get_winning_paths(project, method_fullname):
    state = get_entry_state_of_method(project, method_fullname)
    simgr = project.factory.simgr(state)
    simgr.run()
    paths = simgr.deadended

    # winning paths output a single 'W' on stdout
    winnning_paths = []
    for pp in paths:
        stdout_packets = pp.posix.stdout.content
        read_byte, _ = stdout_packets[0]
        # a winning path is printing 'W'
        pp.solver.add(read_byte == pp.solver.BVV(ord("W"), 8))
        if pp.satisfiable():
            winnning_paths.append(pp)

    return winnning_paths


def get_winning_path(project, method_fullname):
    winning_paths = get_winning_paths(project, method_fullname)
    assert len(winning_paths) != 0
    assert len(winning_paths) == 1
    return winning_paths[0]


def main():
    for k, v in list(globals().items()):
        if k.startswith("test_") and callable(v):
            v()


if __name__ == "__main__":
    # import logging
    # logging.getLogger('cle.backends.soot').setLevel('DEBUG')
    # logging.getLogger('cle.backends.apk').setLevel('DEBUG')
    # logging.getLogger('cle.backends.jar').setLevel('DEBUG')

    # logging.getLogger("angr").setLevel("DEBUG")

    # logging.getLogger("angr.state_plugins").setLevel("INFO")
    # logging.getLogger('angr.state_plugins.javavm_memory').setLevel("DEBUG")
    # logging.getLogger('angr.state_plugins.jni_references').setLevel("DEBUG")
    # logging.getLogger("angr.state_plugins.javavm_classloader").setLevel("DEBUG")

    # logging.getLogger('archinfo.arch_soot').setLevel("DEBUG")
    # logging.getLogger('angr.procedures.java_jni').setLevel("DEBUG")
    # logging.getLogger("angr.sim_procedure").setLevel("DEBUG")
    # logging.getLogger("angr.engines").setLevel("DEBUG")
    # logging.getLogger('angr.simos.JavaVM').setLevel("DEBUG")
    # logging.getLogger('angr.engines.vex').setLevel("DEBUG")
    #
    main()



from ailment.expression import ITE, Convert, BinaryOp, Const

from .base import PeepholeOptimizationExprBase


class SingleBitCondToBoolExpr(PeepholeOptimizationExprBase):
    """
    Convert single-bit conditions to bool expressions
    """

    __slots__ = ()

    NAME = "Convert single-bit conditions to bool expressions (== 0 or == 1)"
    expr_classes = (ITE,)

    def optimize(self, expr: ITE, **kwargs):
        if isinstance(expr.cond, Convert) and expr.cond.to_bits == 1 and expr.cond.from_bits > 1:
            cond_inner = expr.cond.operand
            if isinstance(cond_inner, BinaryOp):
                if (
                    cond_inner.op == "Xor"
                    and isinstance(cond_inner.operands[1], Const)
                    and cond_inner.operands[1].value == 1
                ):
                    # A ^ 1 ==> A == 0
                    new_cond = BinaryOp(
                        None,
                        "CmpEQ",
                        [cond_inner.operands[0], Const(None, None, 0, cond_inner.operands[0].bits)],
                        False,
                        **cond_inner.tags,
                    )
                else:
                    # A ==> A == 1
                    new_cond = BinaryOp(
                        None,
                        "CmpEQ",
                        [cond_inner.operands[0], Const(None, None, 1, cond_inner.operands[0].bits)],
                        False,
                        **cond_inner.tags,
                    )
                return ITE(
                    expr.idx,
                    new_cond,
                    expr.iffalse,
                    expr.iftrue,
                    variable=expr.variable,
                    variable_offset=expr.variable_offset,
                    **expr.tags,
                )

        return expr



import angr
import logging

l = logging.getLogger(name=__name__)


class brk(angr.SimProcedure):
    """
    This implements the brk system call.
    """

    # pylint:disable=arguments-differ

    def run(self, new_brk):
        r = self.state.posix.set_brk(new_brk)
        l.debug("brk(%s) = %s", new_brk, r)
        return r



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("d2d1.dll")
prototypes = \
    {
        #
        'D2D1CreateFactory': SimTypeFunction([SimTypeInt(signed=False, label="D2D1_FACTORY_TYPE"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"debugLevel": SimTypeInt(signed=False, label="D2D1_DEBUG_LEVEL")}, name="D2D1_FACTORY_OPTIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["factoryType", "riid", "pFactoryOptions", "ppIFactory"]),
        #
        'D2D1MakeRotateMatrix': SimTypeFunction([SimTypeFloat(size=32), SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous1": SimStruct({"m11": SimTypeFloat(size=32), "m12": SimTypeFloat(size=32), "m21": SimTypeFloat(size=32), "m22": SimTypeFloat(size=32), "dx": SimTypeFloat(size=32), "dy": SimTypeFloat(size=32)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"_11": SimTypeFloat(size=32), "_12": SimTypeFloat(size=32), "_21": SimTypeFloat(size=32), "_22": SimTypeFloat(size=32), "_31": SimTypeFloat(size=32), "_32": SimTypeFloat(size=32)}, name="_Anonymous2_e__Struct", pack=False, align=None), "m": SimTypeFixedSizeArray(SimTypeFloat(size=32), 6)}, name="<anon>", label="None")}, name="D2D_MATRIX_3X2_F", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["angle", "center", "matrix"]),
        #
        'D2D1MakeSkewMatrix': SimTypeFunction([SimTypeFloat(size=32), SimTypeFloat(size=32), SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous1": SimStruct({"m11": SimTypeFloat(size=32), "m12": SimTypeFloat(size=32), "m21": SimTypeFloat(size=32), "m22": SimTypeFloat(size=32), "dx": SimTypeFloat(size=32), "dy": SimTypeFloat(size=32)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"_11": SimTypeFloat(size=32), "_12": SimTypeFloat(size=32), "_21": SimTypeFloat(size=32), "_22": SimTypeFloat(size=32), "_31": SimTypeFloat(size=32), "_32": SimTypeFloat(size=32)}, name="_Anonymous2_e__Struct", pack=False, align=None), "m": SimTypeFixedSizeArray(SimTypeFloat(size=32), 6)}, name="<anon>", label="None")}, name="D2D_MATRIX_3X2_F", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["angleX", "angleY", "center", "matrix"]),
        #
        'D2D1IsMatrixInvertible': SimTypeFunction([SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous1": SimStruct({"m11": SimTypeFloat(size=32), "m12": SimTypeFloat(size=32), "m21": SimTypeFloat(size=32), "m22": SimTypeFloat(size=32), "dx": SimTypeFloat(size=32), "dy": SimTypeFloat(size=32)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"_11": SimTypeFloat(size=32), "_12": SimTypeFloat(size=32), "_21": SimTypeFloat(size=32), "_22": SimTypeFloat(size=32), "_31": SimTypeFloat(size=32), "_32": SimTypeFloat(size=32)}, name="_Anonymous2_e__Struct", pack=False, align=None), "m": SimTypeFixedSizeArray(SimTypeFloat(size=32), 6)}, name="<anon>", label="None")}, name="D2D_MATRIX_3X2_F", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["matrix"]),
        #
        'D2D1InvertMatrix': SimTypeFunction([SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous1": SimStruct({"m11": SimTypeFloat(size=32), "m12": SimTypeFloat(size=32), "m21": SimTypeFloat(size=32), "m22": SimTypeFloat(size=32), "dx": SimTypeFloat(size=32), "dy": SimTypeFloat(size=32)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"_11": SimTypeFloat(size=32), "_12": SimTypeFloat(size=32), "_21": SimTypeFloat(size=32), "_22": SimTypeFloat(size=32), "_31": SimTypeFloat(size=32), "_32": SimTypeFloat(size=32)}, name="_Anonymous2_e__Struct", pack=False, align=None), "m": SimTypeFixedSizeArray(SimTypeFloat(size=32), 6)}, name="<anon>", label="None")}, name="D2D_MATRIX_3X2_F", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["matrix"]),
        #
        'D2D1CreateDevice': SimTypeFunction([SimTypeBottom(label="IDXGIDevice"), SimTypePointer(SimStruct({"threadingMode": SimTypeInt(signed=False, label="D2D1_THREADING_MODE"), "debugLevel": SimTypeInt(signed=False, label="D2D1_DEBUG_LEVEL"), "options": SimTypeInt(signed=False, label="D2D1_DEVICE_CONTEXT_OPTIONS")}, name="D2D1_CREATION_PROPERTIES", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="ID2D1Device"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dxgiDevice", "creationProperties", "d2dDevice"]),
        #
        'D2D1CreateDeviceContext': SimTypeFunction([SimTypeBottom(label="IDXGISurface"), SimTypePointer(SimStruct({"threadingMode": SimTypeInt(signed=False, label="D2D1_THREADING_MODE"), "debugLevel": SimTypeInt(signed=False, label="D2D1_DEBUG_LEVEL"), "options": SimTypeInt(signed=False, label="D2D1_DEVICE_CONTEXT_OPTIONS")}, name="D2D1_CREATION_PROPERTIES", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="ID2D1DeviceContext"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dxgiSurface", "creationProperties", "d2dDeviceContext"]),
        #
        'D2D1ConvertColorSpace': SimTypeFunction([SimTypeInt(signed=False, label="D2D1_COLOR_SPACE"), SimTypeInt(signed=False, label="D2D1_COLOR_SPACE"), SimTypePointer(SimStruct({"r": SimTypeFloat(size=32), "g": SimTypeFloat(size=32), "b": SimTypeFloat(size=32), "a": SimTypeFloat(size=32)}, name="D2D1_COLOR_F", pack=False, align=None), offset=0)], SimStruct({"r": SimTypeFloat(size=32), "g": SimTypeFloat(size=32), "b": SimTypeFloat(size=32), "a": SimTypeFloat(size=32)}, name="D2D1_COLOR_F", pack=False, align=None), arg_names=["sourceColorSpace", "destinationColorSpace", "color"]),
        #
        'D2D1SinCos': SimTypeFunction([SimTypeFloat(size=32), SimTypePointer(SimTypeFloat(size=32), offset=0), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeBottom(label="Void"), arg_names=["angle", "s", "c"]),
        #
        'D2D1Tan': SimTypeFunction([SimTypeFloat(size=32)], SimTypeFloat(size=32), arg_names=["angle"]),
        #
        'D2D1Vec3Length': SimTypeFunction([SimTypeFloat(size=32), SimTypeFloat(size=32), SimTypeFloat(size=32)], SimTypeFloat(size=32), arg_names=["x", "y", "z"]),
        #
        'D2D1ComputeMaximumScaleFactor': SimTypeFunction([SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous1": SimStruct({"m11": SimTypeFloat(size=32), "m12": SimTypeFloat(size=32), "m21": SimTypeFloat(size=32), "m22": SimTypeFloat(size=32), "dx": SimTypeFloat(size=32), "dy": SimTypeFloat(size=32)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"_11": SimTypeFloat(size=32), "_12": SimTypeFloat(size=32), "_21": SimTypeFloat(size=32), "_22": SimTypeFloat(size=32), "_31": SimTypeFloat(size=32), "_32": SimTypeFloat(size=32)}, name="_Anonymous2_e__Struct", pack=False, align=None), "m": SimTypeFixedSizeArray(SimTypeFloat(size=32), 6)}, name="<anon>", label="None")}, name="D2D_MATRIX_3X2_F", pack=False, align=None), offset=0)], SimTypeFloat(size=32), arg_names=["matrix"]),
        #
        'D2D1GetGradientMeshInteriorPointsFromCoonsPatch': SimTypeFunction([SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32)}, name="D2D_POINT_2F", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPoint0", "pPoint1", "pPoint2", "pPoint3", "pPoint4", "pPoint5", "pPoint6", "pPoint7", "pPoint8", "pPoint9", "pPoint10", "pPoint11", "pTensorPoint11", "pTensorPoint12", "pTensorPoint21", "pTensorPoint22"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.knowledge_plugins.functions"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestFunction(unittest.TestCase):
    def test_function_serialization(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
        cfg = p.analyses.CFG()

        func_main = cfg.kb.functions["main"]
        s = func_main.serialize()

        assert type(s) is bytes
        assert len(s) > 10

        f = angr.knowledge_plugins.Function.parse(s)
        assert func_main.addr == f.addr
        assert func_main.name == f.name
        assert func_main.is_prototype_guessed == f.is_prototype_guessed

    def test_function_definition_application(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
        cfg = p.analyses.CFG()
        func_main: angr.knowledge_plugins.Function = cfg.kb.functions["main"]

        func_main.apply_definition("int main(int argc, char** argv)")

        # Check prototype of function
        assert func_main.prototype.args == [
            angr.sim_type.SimTypeInt().with_arch(p.arch),
            angr.sim_type.SimTypePointer(
                angr.sim_type.SimTypePointer(angr.sim_type.SimTypeChar()).with_arch(p.arch)
            ).with_arch(p.arch),
        ]
        # Check that the default calling convention of the architecture was applied
        assert isinstance(func_main.calling_convention, angr.calling_conventions.DefaultCC[p.arch.name]["Linux"])

        func_main.apply_definition("int main(int argc, char** argv)")

    def test_function_instruction_addr_from_any_addr(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
        cfg = p.analyses.CFG()

        func_main = cfg.kb.functions["main"]

        assert func_main.addr_to_instruction_addr(0x400739) == 0x400739
        assert func_main.addr_to_instruction_addr(0x40073A) == 0x400739
        assert func_main.addr_to_instruction_addr(0x40073D) == 0x400739
        assert func_main.addr_to_instruction_addr(0x400742) == 0x400742
        assert func_main.addr_to_instruction_addr(0x400743) == 0x400742

    def test_function_instruction_size(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
        cfg = p.analyses.CFG()

        func_main = cfg.kb.functions["main"]

        assert func_main.instruction_size(0x40071D) == 1
        assert func_main.instruction_size(0x40071E) == 3
        assert func_main.instruction_size(0x400721) == 4
        assert func_main.instruction_size(0x400725) == 3
        assert func_main.instruction_size(0x400728) == 4
        assert func_main.instruction_size(0x400739) == 5
        assert func_main.instruction_size(0x400742) == 5


if __name__ == "__main__":
    unittest.main()



import logging

from angr.procedures.stubs.format_parser import ScanfFormatParser

l = logging.getLogger(name=__name__)


class scanf(ScanfFormatParser):
    # pylint:disable=arguments-differ,unused-argument

    def run(self, fmt):
        fmt_str = self._parse(fmt)

        # we're reading from stdin so the region is the file's content
        simfd = self.state.posix.get_fd(0)
        if simfd is None:
            return -1

        items = fmt_str.interpret(self.va_arg, simfd=simfd)
        return items



import threading

import rpyc
from rpyc.utils.server import ThreadedServer


class AngrServer(threading.Thread):
    def __init__(self, active_projects=None, port=1234, host="localhost"):
        if active_projects is None:
            active_projects = {}
        super().__init__()
        self.port = port
        self.host = host

        class AngrService(rpyc.Service):
            exposed_projects = active_projects

            def on_connect(self):
                self._conn._config.update(
                    dict(
                        allow_all_attrs=True,
                        allow_pickle=True,
                        allow_getattr=True,
                        allow_setattr=True,
                        allow_delattr=True,
                        import_custom_exceptions=True,
                        instantiate_custom_exceptions=True,
                        instantiate_oldstyle_exceptions=True,
                    )
                )

        self.service = AngrService

    def run(self):
        ThreadedServer(self.service, port=self.port, hostname=self.host).start()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.engines"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestHook(unittest.TestCase):
    def test_mips(self):
        MAIN_END = 0x4007D8
        INNER_LOOP = 0x40069C
        OUTER_LOOP = 0x40076C

        p = angr.Project(os.path.join(test_location, "mips", "test_loops"), auto_load_libs=False)
        output = []

        # hooking by a function decorator
        @p.hook(INNER_LOOP)
        def hook1(_):  # pylint:disable=unused-variable
            output.append(1)

        def hook2(state):
            output.append(2)
            num = state.solver.eval(state.regs.a1)
            string = b"%d " % num
            state.posix.get_fd(1).write_data(state.solver.BVV(string))

        # a manual hook
        p.hook(OUTER_LOOP, hook2, length=0x14)

        s = p.factory.simulation_manager(p.factory.entry_state()).explore(find=[MAIN_END])

        assert len(s.found) == 1
        assert s.found[0].posix.dumps(1) == b"".join(b"%d " % x for x in range(100)) + b"\n"
        assert output == [1] * 100 + [2] * 100
        # print 'Executed %d blocks' % len(s._f.trace)

    def test_zero_length_userhook(self):
        # If a user hook does not overwrite any instruction (length = 0),
        # we should not run the hook twice.
        # jumpkind Ijk_NoHook is used for exactly this purpose.

        class OneTimeHook:
            def __init__(self):
                self.ctr = 0

            def one_time_hook(self, _):
                self.ctr += 1
                if self.ctr > 1:
                    raise Exception("OneTimeHook is executed multiple times.")

        # Amd64
        # 0x0:	mov	qword ptr [1], rax
        # 0x8:	jmp	0x1b
        # 0xa:	mov	qword ptr [2], rax
        # 0x12:	mov	qword ptr [3], rax
        # 0x1a:	ret
        # 0x1b:	jmp	0xa
        shellcode = (
            b"\x48\x89\x04\x25\x01\x00\x00\x00\xeb\x11\x48\x89\x04\x25\x02\x00\x00\x00\x48"
            b"\x89\x04\x25\x03\x00\x00\x00\xc3\xeb\xed"
        )
        proj = angr.load_shellcode(shellcode, arch="amd64")

        proj.hook(0x8, hook=OneTimeHook().one_time_hook, length=0)
        s = proj.factory.simgr()
        s.run()

    def test_nonzero_length_userhook(self):
        # If a user hook overwrites any instruction (length > 0), we should allow the execution of another hook that
        # follows this hook immediately.

        class TwoTimesHook:
            def __init__(self):
                self.addrs = []

            def hook(self, state):
                self.addrs.append(state.addr)

            # Amd64
            # 0x0:	mov	qword ptr [1], rax
            # 0x8:	jmp	0x1b
            # 0xa:	mov	qword ptr [2], rax
            # 0x12:	mov	qword ptr [3], rax
            # 0x1a:	ret
            # 0x1b:	jmp	0xa

        shellcode = (
            b"\x48\x89\x04\x25\x01\x00\x00\x00\xeb\x11\x48\x89\x04\x25\x02\x00\x00\x00\x48"
            b"\x89\x04\x25\x03\x00\x00\x00\xc3\xeb\xed"
        )
        proj = angr.load_shellcode(shellcode, arch="amd64")

        hook = TwoTimesHook()
        proj.hook(0x8, hook=hook.hook, length=2)
        proj.hook(0xA, hook=hook.hook, length=7)
        s = proj.factory.simgr()
        s.run()

        assert hook.addrs == [0x8, 0xA]


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestCfgVexPostprocessor(unittest.TestCase):
    def test_issue_1172(self):
        path = os.path.join(test_location, "x86_64", "cfg_issue_1172")
        p = angr.Project(path, auto_load_libs=False)

        # it should not crash
        _ = p.analyses.CFG()


if __name__ == "__main__":
    unittest.main()



import logging

import claripy
import angr

l = logging.getLogger(name=__name__)

cgc_flag_page_start_addr = 0x4347C000


class allocate(angr.SimProcedure):
    # pylint:disable=arguments-differ,missing-class-docstring

    def run(self, length, is_x, addr):  # pylint:disable=unused-argument
        if self.state.solver.symbolic(length):
            l.warning("Concretizing symbolic length passed to allocate to max_int")

        length = self.state.solver.max_int(length)

        # return code (see allocate() docs)
        r = self.state.solver.ite_cases(
            (
                (length == 0, self.state.cgc.EINVAL),
                (length > self.state.cgc.max_allocation, self.state.cgc.EINVAL),
                (self.state.cgc.addr_invalid(addr), self.state.cgc.EFAULT),
            ),
            self.state.solver.BVV(0, self.state.arch.bits),
        )

        if self.state.solver.max_int(r) != 0:
            # allocate did not succeed. Abort.
            return r

        aligned_length = ((length + 0xFFF) // 0x1000) * 0x1000

        if isinstance(self.state.cgc.allocation_base, int):
            self.state.cgc.allocation_base = self.state.solver.BVV(self.state.cgc.allocation_base, self.state.arch.bits)

        chosen = self.state.cgc.get_max_sinkhole(aligned_length)
        if chosen is None:
            # No sinkhole that can accommodate the requested size exists
            chosen = self.state.cgc.allocation_base - aligned_length
            allocation_base_conc = self.state.solver.eval(self.state.cgc.allocation_base)
            chosen_conc = self.state.solver.eval(chosen)
            if chosen_conc <= cgc_flag_page_start_addr < allocation_base_conc:
                # Chosen memory overlaps with flag page. Add non-overlapping part as a sinkhole and allocate space after
                # the flag page
                sinkhole_size = allocation_base_conc - cgc_flag_page_start_addr - 0x1000
                if sinkhole_size != 0:
                    self.state.cgc.add_sinkhole(cgc_flag_page_start_addr + 0x1000, sinkhole_size)

                chosen = self.state.solver.BVV(cgc_flag_page_start_addr - aligned_length, self.state.arch.bits)
            elif chosen_conc <= self.state.project.loader.max_addr < allocation_base_conc:
                # Chosen memory overlaps with some loaded object
                sinkhole_size = allocation_base_conc - self.state.project.loader.max_addr
                if sinkhole_size != 0:
                    self.state.cgc.add_sinkhole(self.state.project.loader.max_addr, sinkhole_size)

                chosen = self.state.solver.BVV(
                    self.state.project.loader.min_addr - aligned_length, self.state.arch.bits
                )

            self.state.cgc.allocation_base = chosen

        self.state.memory.store(
            addr, chosen, size=self.state.arch.bytes, condition=self.state.solver.And(addr != 0), endness="Iend_LE"
        )

        # PROT_READ | PROT_WRITE default
        permissions = self.state.solver.BVV(1 | 2, 3)
        permissions |= self.state.solver.If(is_x != 0, claripy.BVV(4, 3), claripy.BVV(0, 3))

        chosen_conc = self.state.solver.eval(chosen)
        l.debug("Allocating [%#x, %#x]", chosen_conc, chosen_conc + aligned_length - 1)
        self.state.memory.map_region(chosen_conc, aligned_length, permissions)
        return r



import angr


class strcasecmp(angr.SimProcedure):
    # pylint:disable=arguments-differ

    def run(self, a_addr, b_addr):
        strlen = angr.SIM_PROCEDURES["libc"]["strlen"]

        a_strlen = self.inline_call(strlen, a_addr)
        b_strlen = self.inline_call(strlen, b_addr)
        maxlen = self.state.solver.BVV(max(a_strlen.max_null_index, b_strlen.max_null_index), self.state.arch.bits)

        strncmp = self.inline_call(
            angr.SIM_PROCEDURES["libc"]["strncmp"],
            a_addr,
            b_addr,
            maxlen,
            a_len=a_strlen,
            b_len=b_strlen,
            ignore_case=True,
        )
        return strncmp.ret_expr



import logging

from capstone.x86_const import X86_REG_RIP

from pyvex.stmt import IMark

from .resolver import IndirectJumpResolver

l = logging.getLogger(name=__name__)


class AMD64ElfGotResolver(IndirectJumpResolver):
    """
    A timeless indirect jump resolver that resolves GOT entries on AMD64 ELF binaries.
    """

    def __init__(self, project):
        super().__init__(project, timeless=True)

    def filter(self, cfg, addr, func_addr, block, jumpkind):
        if jumpkind != "Ijk_Call":
            return False
        return True

    def resolve(  # pylint:disable=unused-argument
        self, cfg, addr, func_addr, block, jumpkind, func_graph_complete: bool = True, **kwargs
    ):
        # Find the address and size of the last instruction
        last_insn_addr = None
        last_insn_size = None
        for stmt in reversed(block.statements):
            if isinstance(stmt, IMark):
                last_insn_addr = stmt.addr
                last_insn_size = stmt.len
                break

        if last_insn_addr is None:
            # Cannot find the last instruction
            return False, []

        # lift one instruction
        insn = self.project.factory.block(last_insn_addr, size=last_insn_size).capstone.insns[-1]
        opnd = insn.insn.operands[0]
        # Must be of the form: call [rip + 0xABCD]
        if not (opnd.mem and opnd.mem.disp and opnd.mem.base == X86_REG_RIP and not opnd.mem.index):
            return False, []

        disp = insn.insn.disp
        slot = disp + insn.address + insn.size
        target = cfg._fast_memory_load_pointer(slot)
        if target is None:
            l.warning("Address %# is not mapped.", slot)
            return False, []

        if self.project.loader.find_symbol(target):
            return True, [target]

        if not self.project.is_hooked(target):
            return False, []

        dest = self.project.hooked_by(target)
        l.debug("Resolved target to %s", dest.display_name)
        return True, [target]



#!/usr/bin/env python3
# pylint:disable=no-self-use
__package__ = __package__ or "tests.knowledge_plugins"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class PatchTests(unittest.TestCase):
    """
    Basic PatchManager tests
    """

    def test_patch_vulnerable_fauxware_amd64(self):
        binpath = os.path.join(test_location, "x86_64", "vulns", "vulnerable_fauxware")
        proj = angr.Project(binpath, auto_load_libs=False)

        proj.kb.patches.add_patch(0x40094C, b"\x0a")
        patched = proj.kb.patches.apply_patches_to_binary()

        # manual patch
        with open(binpath, "rb") as f:
            binary_data = f.read()
        binary_data = binary_data[:0x94C] + b"\x0a" + binary_data[0x94D:]

        assert patched == binary_data

    def test_block_factory_returns_patched_bytes(self):
        binpath = os.path.join(test_location, "x86_64", "fauxware")
        proj = angr.Project(binpath, auto_load_libs=False)

        addr = 0x4007D3
        patch_bytes = proj.arch.keystone.asm("inc rax; leave; ret", addr, as_bytes=True)[0]
        proj.kb.patches.add_patch(addr, patch_bytes)

        b = proj.factory.block(addr)
        assert b.bytes == patch_bytes


if __name__ == "__main__":
    unittest.main()
